# -*- coding: utf-8 -*-
# Copyright (C) 2021-2022  Nexedi SA and Contributors.
#                          Kirill Smelkov <kirr@nexedi.com>
#
# This program is free software: you can Use, Study, Modify and Redistribute
# it under the terms of the GNU General Public License version 3, or (at your
# option) any later version, as published by the Free Software Foundation.
#
# You can also Link and Combine this program with other software covered by
# the terms of any of the Free Software licenses or any of the Open Source
# Initiative approved licenses and Convey the resulting work. Corresponding
# source of such a combination shall include the source code for all other
# software used.
#
# This program is distributed WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See COPYING file for full licensing terms.
# See https://www.nexedi.com/licensing for rationale and options.
"""Package syscall mirrors Go package syscall.

- SIG* provide constants for signals.

See also https://golang.org/pkg/syscall for Go syscall package documentation.
"""

from __future__ import print_function, absolute_import


# create signal constants wrapped into os.Signal
# for example syscall.SIGTERM = _PySignal_from_int(SIGTERM)
def _():
    from golang._os import _PySignal_from_int
    import signal as _pysig
    for attr in dir(_pysig):
        if attr.startswith("SIG") and not attr.startswith("SIG_"):
            signo = getattr(_pysig, attr)
            sig = _PySignal_from_int(signo)
            globals()[attr] = sig
_(); del _
